# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TurbineGovernorDynamics import TurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovGAST1(TurbineGovernorDynamics):
    """
    Modified single shaft gas turbine.

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    r: Permanent droop (R) (>0).  Typical value = 0,04.
    t1: Governor mechanism time constant (T1) (>= 0).  T1 represents the natural valve positioning time constant of the
      governor for small disturbances, as seen when rate limiting is not in effect.  Typical value = 0,5.
    t2: Turbine power time constant (T2) (>= 0). T2 represents delay due to internal energy storage of the gas turbine
      engine. T2 can be used to give a rough approximation to the delay associated with acceleration of the
      compressor spool of a multi-shaft engine, or with the compressibility of gas in the plenum of the free
      power turbine of an aero-derivative unit, for example.  Typical value = 0,5.
    t3: Turbine exhaust temperature time constant (T3) (>= 0).  T3 represents delay in the exhaust temperature and load
      limiting system. Typical value = 3.
    lmax: Ambient temperature load limit (Lmax).  Lmax is the turbine power output corresponding to the limiting exhaust
      gas temperature.  Typical value = 1.
    kt: Temperature limiter gain (Kt).  Typical value = 3.
    vmax: Maximum turbine power, PU of MWbase (Vmax) (> GovGAST1.vmin).  Typical value = 1.
    vmin: Minimum turbine power, PU of MWbase (Vmin) (< GovGAST1.vmax).  Typical value = 0.
    fidle: Fuel flow at zero power output (Fidle).  Typical value = 0,18.
    rmax: Maximum fuel valve opening rate (Rmax).  Unit = PU / s.  Typical value = 1.
    loadinc: Valve position change allowed at fast rate (Loadinc).  Typical value = 0,05.
    tltr: Valve position averaging time constant (Tltr) (>= 0).  Typical value = 10.
    ltrate: Maximum long term fuel valve opening rate (Ltrate).  Typical value = 0,02.
    a: Turbine power time constant numerator scale factor (a).  Typical value = 0,8.
    b: Turbine power time constant denominator scale factor (b) (>0).  Typical value = 1.
    db1: Intentional dead-band width (db1).  Unit = Hz.  Typical value = 0.
    eps: Intentional db hysteresis (eps).  Unit = Hz.  Typical value = 0.
    db2: Unintentional dead-band (db2).  Unit = MW.  Typical value = 0.
    gv1: Nonlinear gain point 1, PU gv (Gv1).  Typical value = 0.
    pgv1: Nonlinear gain point 1, PU power (Pgv1).  Typical value = 0.
    gv2: Nonlinear gain point 2,PU gv (Gv2).  Typical value = 0.
    pgv2: Nonlinear gain point 2, PU power (Pgv2).  Typical value = 0.
    gv3: Nonlinear gain point 3, PU gv (Gv3).  Typical value = 0.
    pgv3: Nonlinear gain point 3, PU power (Pgv3).  Typical value = 0.
    gv4: Nonlinear gain point 4, PU gv (Gv4).  Typical value = 0.
    pgv4: Nonlinear gain point 4, PU power (Pgv4).  Typical value = 0.
    gv5: Nonlinear gain point 5, PU gv (Gv5).  Typical value = 0.
    pgv5: Nonlinear gain point 5, PU power (Pgv5).  Typical value = 0.
    gv6: Nonlinear gain point 6, PU gv (Gv6).  Typical value = 0.
    pgv6: Nonlinear gain point 6, PU power (Pgv6).  Typical value = 0.
    ka: Governor gain (Ka).  Typical value = 0.
    t4: Governor lead time constant (T4) (>= 0).  Typical value = 0.
    t5: Governor lag time constant (T5) (>= 0).  If = 0, entire gain and lead-lag block is bypassed.  Typical value = 0.
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    r: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    lmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kt: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fidle: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    loadinc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tltr: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ltrate: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    db1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eps: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    db2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .RegulatingCondEq import RegulatingCondEq


@dataclass(config=DataclassConfig)
class StaticVarCompensator(RegulatingCondEq):
    """
    A facility for providing variable and controllable shunt reactive power. The SVC typically consists of a stepdown
      transformer, filter, thyristor-controlled reactor, and thyristor-switched capacitor arms.  The SVC may operate
      in fixed MVar output mode or in voltage control mode. When in voltage control mode, the output of the SVC will
      be proportional to the deviation of voltage at the controlled bus from the voltage setpoint.  The SVC
      characteristic slope defines the proportion.  If the voltage at the controlled bus is equal to the voltage
      setpoint, the SVC MVar output is zero.

    capacitiveRating: Capacitive reactance at maximum capacitive reactive power.  Shall always be positive.
    inductiveRating: Inductive reactance at maximum inductive reactive power.  Shall always be negative.
    slope: The characteristics slope of an SVC defines how the reactive power output changes in proportion to the
      difference between the regulated bus voltage and the voltage setpoint. The attribute shall be a
      positive value or zero.
    sVCControlMode: SVC control mode.
    voltageSetPoint: The reactive power output of the SVC is proportional to the difference between the voltage at the
      regulated bus and the voltage setpoint.  When the regulated bus voltage is equal to the
      voltage setpoint, the reactive power output is zero.
    q: Reactive power injection. Load sign convention is used, i.e. positive sign means flow out from a node. Starting
      value for a steady state solution.
    StaticVarCompensatorDynamics: Static Var Compensator dynamics model used to describe dynamic behaviour of this
      Static Var Compensator.
    """

    capacitiveRating: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    inductiveRating: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    slope: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    sVCControlMode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    voltageSetPoint: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    q: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # StaticVarCompensatorDynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SSH,
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class LoadMotor(IdentifiedObject):
    """
    Aggregate induction motor load. This model is used to represent a fraction of an ordinary load as "induction motor
      load".  It allows a load that is treated as an ordinary constant power in power flow analysis to be
      represented by an induction motor in dynamic simulation. This model is intended for representation of
      aggregations of many motors dispersed through a load represented at a high voltage bus but where there is no
      information on the characteristics of individual motors. Either a "one-cage" or "two-cage" model of the
      induction machine can be modelled. Magnetic saturation is not modelled. This model treats a fraction of the
      constant power part of a load as a motor. During initialisation, the initial power drawn by the motor is set
      equal to Pfrac times the constant P part of the static load.  The remainder of the load is left as a static
      load. The reactive power demand of the motor is calculated during initialisation as a function of voltage at
      the load bus. This reactive power demand can be less than or greater than the constant Q component of the
      load.  If the motor's reactive demand is greater than the constant Q component of the load, the model inserts
      a shunt capacitor at the terminal of the motor to bring its reactive demand down to equal the constant Q
      reactive load.   If an induction motor load model and a static load model are both present for a load, the
      motor Pfrac is assumed to be subtracted from the power flow constant P load before the static load model is
      applied.  The remainder of the load, if any, is then represented by the static load model.

    LoadAggregate: Aggregate load to which this aggregate motor (dynamic) load belongs.
    pfrac: Fraction of constant-power load to be represented by this motor model (Pfrac) (>= 0,0 and <= 1,0).  Typical
      value = 0,3.
    lfac: Loading factor (Lfac). The ratio of initial P to motor MVA base.  Typical value = 0,8.
    ls: Synchronous reactance (Ls).  Typical value = 3,2.
    lp: Transient reactance (Lp).  Typical value = 0,15.
    lpp: Subtransient reactance (Lpp).  Typical value = 0,15.
    ra: Stator resistance (Ra).  Typical value = 0.
    tpo: Transient rotor time constant (Tpo) (>= 0).  Typical value = 1.
    tppo: Subtransient rotor time constant (Tppo) (>= 0).  Typical value = 0,02.
    h: Inertia constant (H) (>= 0).  Typical value = 0,4.
    d: Damping factor (D).  Unit = delta P/delta speed.  Typical value = 2.
    vt: Voltage threshold for tripping (Vt).  Typical value = 0,7.
    tv: Voltage trip pickup time (Tv) (>= 0).  Typical value = 0,1.
    tbkr: Circuit breaker operating time (Tbkr) (>= 0).  Typical value = 0,08.
    """

    LoadAggregate: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    pfrac: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    lfac: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ls: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    lp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    lpp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ra: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpo: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tppo: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    h: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    d: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vt: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tv: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tbkr: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEST1A(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type ST1A model. This model represents systems in which excitation power is supplied through a
      transformer from the generator terminals (or the unit's auxiliary bus) and is regulated by a controlled
      rectifier.  The maximum exciter voltage available from such systems is directly related to the generator
      terminal voltage. Reference: IEEE 421.5-2005, 7.1.

    ilr: Exciter output current limit reference (ILR).  Typical value = 0.
    ka: Voltage regulator gain (KA) (> 0).  Typical value = 190.
    kc: Rectifier loading factor proportional to commutating reactance (KC) (>= 0). Typical value = 0,08.
    kf: Excitation control system stabilizer gains (KF) (>= 0).  Typical value = 0.
    klr: Exciter output current limiter gain (KLR).  Typical value = 0.
    pssin: Selector of the Power System Stabilizer (PSS) input (PSSin). true = PSS input (Vs) added to error signal
      false = PSS input (Vs) added to voltage regulator output. Typical value = true.
    ta: Voltage regulator time constant (TA) (>= 0).  Typical value = 0.
    tb: Voltage regulator time constant (TB) (>= 0).  Typical value = 10.
    tb1: Voltage regulator time constant (TB1) (>= 0).  Typical value = 0.
    tc: Voltage regulator time constant (TC) (>= 0).  Typical value = 1.
    tc1: Voltage regulator time constant (TC1) (>= 0).  Typical value = 0.
    tf: Excitation control system stabilizer time constant (TF) (>= 0).  Typical value = 1.
    uelin: Selector of the connection of the UEL input (UELin).  Typical value = ignoreUELsignal.
    vamax: Maximum voltage regulator output (VAMAX) (> 0).  Typical value = 14,5.
    vamin: Minimum voltage regulator output (VAMIN) (< 0).  Typical value = -14,5.
    vimax: Maximum voltage regulator input limit (VIMAX) (> 0).  Typical value = 999.
    vimin: Minimum voltage regulator input limit (VIMIN) (< 0).  Typical value = -999.
    vrmax: Maximum voltage regulator outputs (VRMAX) (> 0).  Typical value = 7,8.
    vrmin: Minimum voltage regulator outputs (VRMIN) (< 0).  Typical value = -6,7.
    """

    ilr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    klr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pssin: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uelin: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vimin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class Pss1(PowerSystemStabilizerDynamics):
    """
    Italian PSS with three inputs (speed, frequency, power).

    komega: Shaft speed power input gain (Komega).  Typical value = 0.
    kf: Frequency power input gain (KF).  Typical value = 5.
    kpe: Electric power input gain (KPE).  Typical value = 0,3.
    pmin: Minimum power PSS enabling (Pmin).  Typical value = 0,25.
    ks: PSS gain (Ks).  Typical value = 1.
    vsmn: Stabilizer output maximum limit (VSMN).  Typical value = -0,06.
    vsmx: Stabilizer output minimum limit (VSMX).  Typical value = 0,06.
    tpe: Electric power filter time constant (TPE) (>= 0).  Typical value = 0,05.
    t5: Washout (T5) (>= 0).  Typical value = 3,5.
    t6: Filter time constant (T6) (>= 0).  Typical value = 0.
    t7: Lead/lag time constant (T7) (>= 0). If = 0, both blocks are bypassed.  Typical value = 0.
    t8: Lead/lag time constant (T8) (>= 0).  Typical value = 0.
    t9: Lead/lag time constant (T9) (>= 0).  If = 0, both blocks are bypassed.  Typical value = 0.
    t10: Lead/lag time constant (T10) (>= 0).  Typical value = 0.
    vadat: Signal selector (VADAT). true = closed (generator power is greater than Pmin) false = open (Pe is smaller
      than Pmin). Typical value = true.
    """

    komega: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpe: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsmn: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsmx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpe: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t7: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t8: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t9: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t10: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vadat: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class ControlAreaGeneratingUnit(IdentifiedObject):
    """
    A control area generating unit. This class is needed so that alternate control area definitions may include the same
      generating unit.   It should be noted that only one instance within a control area should reference a specific
      generating unit.

    ControlArea: The parent control area for the generating unit specifications.
    GeneratingUnit: The generating unit specified for this control area.  Note that a control area should include a
      GeneratingUnit only once.
    """

    ControlArea: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    GeneratingUnit: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .DynamicsFunctionBlock import DynamicsFunctionBlock


@dataclass(config=DataclassConfig)
class WindPlantDynamics(DynamicsFunctionBlock):
    """
    Parent class supporting relationships to wind turbines type 3 and type 4 and wind plant IEC and user-defined wind
      plants including their control models.

    RemoteInputSignal: The remote signal with which this power plant is associated.
    WindTurbineType3or4Dynamics: The wind turbine type 3 or type 4 associated with this wind plant.
    """

    RemoteInputSignal: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:1..n in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3or4Dynamics : list = Field(default_factory=list, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcBBC(ExcitationSystemDynamics):
    """
    Transformer fed static excitation system (static with ABB regulator). This model represents a static excitation
      system in which a gated thyristor bridge fed by a transformer at the main generator terminals feeds the main
      generator directly.

    t1: Controller time constant (T1) (>= 0).  Typical value = 6.
    t2: Controller time constant (T2) (>= 0).  Typical value = 1.
    t3: Lead/lag time constant (T3) (>= 0).  If = 0, block is bypassed.  Typical value = 0,05.
    t4: Lead/lag time constant (T4) (>= 0).  If = 0, block is bypassed.  Typical value = 0,01.
    k: Steady state gain (K) (not = 0).  Typical value = 300.
    vrmin: Minimum control element output (Vrmin) (< ExcBBC.vrmax).  Typical value = -5.
    vrmax: Maximum control element output (Vrmax) (> ExcBBC.vrmin).  Typical value = 5.
    efdmin: Minimum open circuit exciter voltage (Efdmin) (< ExcBBC.efdmax).  Typical value = -5.
    efdmax: Maximum open circuit exciter voltage (Efdmax) (> ExcBBC.efdmin).  Typical value = 5.
    xe: Effective excitation transformer reactance (Xe) (>= 0).  Xe models the regulation of the transformer/rectifier
      unit.  Typical value = 0,05.
    switch: Supplementary signal routing selector (switch). true = Vs connected to 3rd summing point false =  Vs
      connected to 1st summing point (see diagram). Typical value = false.
    """

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efdmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efdmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xe: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    switch: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class WindContCurrLimIEC(IdentifiedObject):
    """
    Current limitation model.  The current limitation model combines the physical limits and the control limits.
      Reference: IEC 61400-27-1:2015, 5.6.5.8.

    imax: Maximum continuous current at the wind turbine terminals (imax). It is a type-dependent parameter.
    imaxdip: Maximum current during voltage dip at the wind turbine terminals (imaxdip). It is a project-dependent
      parameter.
    kpqu: Partial derivative of reactive current limit (Kpqu) versus voltage. It is a type-dependent parameter.
    mdfslim: Limitation of type 3 stator current (MDFSLim). MDFSLim = 1 for wind turbines type 4. It is a type-dependent
      parameter. false= total current limitation (0 in the IEC model) true=stator current limitation (1 in
      the IEC model).
    mqpri: Prioritisation of Q control during UVRT (Mqpri). It is a project-dependent parameter. true = reactive power
      priority (1 in the IEC model) false = active power priority (0 in the IEC model).
    tufiltcl: Voltage measurement filter time constant (Tufiltcl) (>= 0). It is a type-dependent parameter.
    upqumax: Wind turbine voltage in the operation point where zero reactive current can be delivered (upqumax). It is a
      type-dependent parameter.
    WindTurbineType3or4IEC: Wind turbine type 3 or type 4 model with which this wind control current limitation model is
      associated.
    WindDynamicsLookupTable: The wind dynamics lookup table associated with this current control limitation model.
    """

    imax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    imaxdip: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpqu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    mdfslim: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    mqpri: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    tufiltcl: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    upqumax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3or4IEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    # *Association not used*
    # Type M:1..n in CIM  # pylint: disable-next=line-too-long
    # WindDynamicsLookupTable : list = Field(default_factory=list, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcAVR3(ExcitationSystemDynamics):
    """
    Italian excitation system. It represents an exciter dynamo and electric regulator.

    ka: AVR gain (KA).  Typical value = 100.
    vrmn: Minimum AVR output (VRMN).  Typical value = -7,5.
    vrmx: Maximum AVR output (VRMX).  Typical value = 7,5.
    t1: AVR time constant (T1) (>= 0).  Typical value = 20.
    t2: AVR time constant (T2) (>= 0).  Typical value = 1,6.
    t3: AVR time constant (T3) (>= 0).  Typical value = 0,66.
    t4: AVR time constant (T4) (>= 0).  Typical value = 0,07.
    te: Exciter time constant (TE) (>= 0).  Typical value = 1.
    e1: Field voltage value 1 (E1).  Typical value = 4,18.
    se1: Saturation factor at E1 (S[E1]).  Typical value = 0,1.
    e2: Field voltage value 2 (E2).  Typical value = 3,14.
    se2: Saturation factor at E2 (S[E2]).  Typical value = 0,03.
    """

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmn: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    e1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    se1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    e2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    se2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class DiagramObjectPoint(Base):
    """
    A point in a given space defined by 3 coordinates and associated to a diagram object.  The coordinates may be
      positive or negative as the origin does not have to be in the corner of a diagram.

    DiagramObject: The diagram object with which the points are associated.
    DiagramObjectGluePoint: The `glue` point to which this point is associated.
    sequenceNumber: The sequence position of the point, used for defining the order of points for diagram objects acting
      as a polyline or polygon with more than one point. The attribute shall be a positive value.
    xPosition: The X coordinate of this point.
    yPosition: The Y coordinate of this point.
    zPosition: The Z coordinate of this point.
    """

    DiagramObject: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DL,
        ],
    )

    DiagramObjectGluePoint: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DL,
        ],
    )

    sequenceNumber: int = Field(
        default=0,
        in_profiles=[
            Profile.DL,
        ],
    )

    xPosition: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DL,
        ],
    )

    yPosition: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DL,
        ],
    )

    zPosition: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DL,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DL,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .RotatingMachine import RotatingMachine


@dataclass(config=DataclassConfig)
class AsynchronousMachine(RotatingMachine):
    """
    A rotating machine whose shaft rotates asynchronously with the electrical field.  Also known as an induction machine
      with no external connection to the rotor windings, e.g. squirrel-cage induction machine.

    nominalFrequency: Nameplate data indicates if the machine is 50 Hz or 60 Hz.
    nominalSpeed: Nameplate data.  Depends on the slip and number of pole pairs.
    converterFedDrive: Indicates whether the machine is a converter fed drive. Used for short circuit data exchange
      according to IEC 60909.
    efficiency: Efficiency of the asynchronous machine at nominal operation as a percentage. Indicator for converter
      drive motors. Used for short circuit data exchange according to IEC 60909.
    iaIrRatio: Ratio of locked-rotor current to the rated current of the motor (Ia/Ir). Used for short circuit data
      exchange according to IEC 60909.
    polePairNumber: Number of pole pairs of stator. Used for short circuit data exchange according to IEC 60909.
    ratedMechanicalPower: Rated mechanical power (Pr in IEC 60909-0). Used for short circuit data exchange according to
      IEC 60909.
    reversible: Indicates for converter drive motors if the power can be reversible. Used for short circuit data
      exchange according to IEC 60909.
    rxLockedRotorRatio: Locked rotor ratio (R/X). Used for short circuit data exchange according to IEC 60909.
    asynchronousMachineType: Indicates the type of Asynchronous Machine (motor or generator).
    AsynchronousMachineDynamics: Asynchronous machine dynamics model used to describe dynamic behaviour of this
      asynchronous machine.
    """

    nominalFrequency: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    nominalSpeed: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    converterFedDrive: bool = Field(
        default=False,
        in_profiles=[
            Profile.SC,
        ],
    )

    efficiency: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    iaIrRatio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    polePairNumber: int = Field(
        default=0,
        in_profiles=[
            Profile.SC,
        ],
    )

    ratedMechanicalPower: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    reversible: bool = Field(
        default=False,
        in_profiles=[
            Profile.SC,
        ],
    )

    rxLockedRotorRatio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    asynchronousMachineType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SSH,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # AsynchronousMachineDynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SC,
            Profile.SSH,
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEDC1A(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type DC1A model. This model represents field-controlled DC commutator exciters with continuously
      acting voltage regulators (especially the direct-acting rheostatic, rotating amplifier, and magnetic amplifier
      types).  Because this model has been widely implemented by the industry, it is sometimes used to represent
      other types of systems when detailed data for them are not available or when a simplified model is required.
      Reference: IEEE 421.5-2005, 5.1.

    ka: Voltage regulator gain (KA) (> 0).  Typical value = 46.
    ta: Voltage regulator time constant (TA) (> 0).  Typical value = 0,06.
    tb: Voltage regulator time constant (TB) (>= 0).  Typical value = 0.
    tc: Voltage regulator time constant (TC) (>= 0).  Typical value = 0.
    vrmax: Maximum voltage regulator output (VRMAX) (> ExcIEEEDC1A.vrmin).  Typical value = 1.
    vrmin: Minimum voltage regulator output (VRMIN) (< 0 and < ExcIEEEDC1A.vrmax).  Typical value = -0,9.
    ke: Exciter constant related to self-excited field (KE).  Typical value = 0.
    te: Exciter time constant, integration rate associated with exciter control (TE) (> 0).  Typical value = 0,46.
    kf: Excitation control system stabilizer gain (KF) (>= 0).  Typical value = 0.1.
    tf: Excitation control system stabilizer time constant (TF) (> 0).  Typical value = 1.
    efd1: Exciter voltage at which exciter saturation is defined (EFD1) (> 0).  Typical value = 3,1.
    seefd1: Exciter saturation function value at the corresponding exciter voltage, EFD1 (SE[EFD1]) (>= 0).  Typical
      value = 0.33.
    efd2: Exciter voltage at which exciter saturation is defined (EFD2) (> 0).  Typical value = 2,3.
    seefd2: Exciter saturation function value at the corresponding exciter voltage, EFD2 (SE[EFD2]) (>= 0).  Typical
      value = 0,1.
    uelin: UEL input (uelin). true = input is connected to the HV gate false = input connects to the error signal.
      Typical value = true.
    exclim: (exclim).  IEEE standard is ambiguous about lower limit on exciter output. true = a lower limit of zero is
      applied to integrator output false = a lower limit of zero is not applied to integrator output.
      Typical value = true.
    """

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seefd1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seefd2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uelin: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    exclim: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .WindTurbineType3or4IEC import WindTurbineType3or4IEC


@dataclass(config=DataclassConfig)
class WindTurbineType3IEC(WindTurbineType3or4IEC):
    """
    Parent class supporting relationships to IEC wind turbines type 3 including their control models.

    WindAeroOneDimIEC: Wind aerodynamic model associated with this wind generator type 3 model.
    WindAeroTwoDimIEC: Wind aerodynamic model associated with this wind turbine type 3 model.
    WindContPitchAngleIEC: Wind control pitch angle model associated with this wind turbine type 3.
    WindContPType3IEC: Wind control P type 3 model associated with this wind turbine type 3 model.
    WindGenType3IEC: Wind generator type 3 model associated with this wind turbine type 3 model.
    WindMechIEC: Wind mechanical model associated with this wind turbine type 3 model.
    """

    WindAeroOneDimIEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    WindAeroTwoDimIEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    WindContPitchAngleIEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    WindContPType3IEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    WindGenType3IEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    WindMechIEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class EnergyArea(IdentifiedObject):
    """
    Describes an area having energy production or consumption.  Specializations are intended to support the load
      allocation function as typically required in energy management systems or planning studies to allocate
      hypothesized load levels to individual load points for power flow analysis.  Often the energy area can be
      linked to both measured and forecast load levels.

    ControlArea: The control area specification that is used for the load forecast.
    """

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # ControlArea : Optional[str] = Field(default=None, in_profiles = [Profile.EQ, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .SynchronousMachineDynamics import SynchronousMachineDynamics


@dataclass(config=DataclassConfig)
class SynchronousMachineDetailed(SynchronousMachineDynamics):
    """
    All synchronous machine detailed types use a subset of the same data parameters and input/output variables.   The
      several variations differ in the following ways: - the number of  equivalent windings that are included; - the
      way in which saturation is incorporated into the model; - whether or not "subtransient saliency" (X''q not =
      X''d) is represented. It is not necessary for each simulation tool to have separate models for each of the
      model types.  The same model can often be used for several types by alternative logic within the model.  Also,
      differences in saturation representation might not result in significant model performance differences so
      model substitutions are often acceptable.

    saturationFactorQAxis: Quadrature-axis saturation factor at rated terminal voltage (S1q) (>= 0). Typical value =
      0,02.
    saturationFactor120QAxis: Quadrature-axis saturation factor at 120% of rated terminal voltage (S12q) (>=
      SynchonousMachineDetailed.saturationFactorQAxis).  Typical value = 0,12.
    efdBaseRatio: Ratio (exciter voltage/generator voltage) of Efd bases of exciter and generator models (> 0). Typical
      value = 1.
    ifdBaseType: Excitation base system mode. It should be equal to the value of WLMDV given by the user. WLMDV is the
      PU ratio between the field voltage and the excitation current: Efd = WLMDV x Ifd. Typical value =
      ifag.
    """

    saturationFactorQAxis: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    saturationFactor120QAxis: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efdBaseRatio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ifdBaseType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemResource import PowerSystemResource


@dataclass(config=DataclassConfig)
class RegulatingControl(PowerSystemResource):
    """
    Specifies a set of equipment that works together to control a power system quantity such as voltage or flow.  Remote
      bus voltage control is possible by specifying the controlled terminal located at some place remote from the
      controlling equipment. The specified terminal shall be associated with the connectivity node of the controlled
      point.  The most specific subtype of RegulatingControl shall be used in case such equipment participate in the
      control, e.g. TapChangerControl for tap changers. For flow control, load sign convention is used, i.e.
      positive sign means flow out from a TopologicalNode (bus) into the conducting equipment. The attribute
      minAllowedTargetValue and maxAllowedTargetValue are required in the following cases: - For a power generating
      module operated in power factor control mode to specify maximum and minimum power factor values; - Whenever it
      is necessary to have an off center target voltage for the tap changer regulator. For instance, due to long
      cables to off shore wind farms and the need to have a simpler setup at the off shore transformer platform, the
      voltage is controlled from the land at the connection point for the off shore wind farm. Since there usually
      is a voltage rise along the cable, there is typical and overvoltage of up 3-4 kV compared to the on shore
      station. Thus in normal operation the tap changer on the on shore station is operated with a target set point,
      which is in the lower parts of the dead band. The attributes minAllowedTargetValue and maxAllowedTargetValue
      are not related to the attribute targetDeadband and thus they are not treated as an alternative of the
      targetDeadband. They are needed due to limitations in the local substation controller. The attribute
      targetDeadband is used to prevent the power flow from move the tap position in circles (hunting) that is to be
      used regardless of the attributes minAllowedTargetValue and maxAllowedTargetValue.

    RegulationSchedule: Schedule for this regulating control.
    RegulatingCondEq: The equipment that participates in this regulating control scheme.
    mode: The regulating control mode presently available.  This specification allows for determining the kind of
      regulation without need for obtaining the units from a schedule.
    Terminal: The terminal associated with this regulating control.  The terminal is associated instead of a node, since
      the terminal could connect into either a topological node or a connectivity node.  Sometimes it is
      useful to model regulation at a terminal of a bus bar object.
    discrete: The regulation is performed in a discrete mode. This applies to equipment with discrete controls, e.g. tap
      changers and shunt compensators.
    enabled: The flag tells if regulation is enabled.
    targetDeadband: This is a deadband used with discrete control to avoid excessive update of controls like tap
      changers and shunt compensator banks while regulating.  The units of those appropriate for the
      mode. The attribute shall be a positive value or zero. If RegulatingControl.discrete is set to
      `false`, the RegulatingControl.targetDeadband is to be ignored. Note that for instance, if the
      targetValue is 100 kV and the targetDeadband is 2 kV the range is from 99 to 101 kV.
    targetValue: The target value specified for case input.   This value can be used for the target value without the
      use of schedules. The value has the units appropriate to the mode attribute.
    targetValueUnitMultiplier: Specify the multiplier for used for the targetValue.
    maxAllowedTargetValue: Maximum allowed target value (RegulatingControl.targetValue).
    minAllowedTargetValue: Minimum allowed target value (RegulatingControl.targetValue).
    """

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # RegulationSchedule : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # RegulatingCondEq : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    mode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    Terminal: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    discrete: bool = Field(
        default=False,
        in_profiles=[
            Profile.SSH,
        ],
    )

    enabled: bool = Field(
        default=False,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetDeadband: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetValue: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetValueUnitMultiplier: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SSH,
        ],
    )

    maxAllowedTargetValue: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    minAllowedTargetValue: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SSH,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .RotatingMachine import RotatingMachine


@dataclass(config=DataclassConfig)
class SynchronousMachine(RotatingMachine):
    """
    An electromechanical device that operates with shaft rotating synchronously with the network. It is a single machine
      operating either as a generator or synchronous condenser or pump.

    InitialReactiveCapabilityCurve: The default reactive capability curve for use by a synchronous machine.
    maxQ: Maximum reactive power limit. This is the maximum (nameplate) limit for the unit.
    minQ: Minimum reactive power limit for the unit.
    qPercent: Part of the coordinated reactive control that comes from this machine. The attribute is used as a
      participation factor not necessarily summing up to 100% for the participating devices in the
      control.
    type: Modes that this synchronous machine can operate in.
    earthing: Indicates whether or not the generator is earthed. Used for short circuit data exchange according to IEC
      60909.
    earthingStarPointR: Generator star point earthing resistance (Re). Used for short circuit data exchange according to
      IEC 60909.
    earthingStarPointX: Generator star point earthing reactance (Xe). Used for short circuit data exchange according to
      IEC 60909.
    ikk: Steady-state short-circuit current (in A for the profile) of generator with compound excitation during 3-phase
      short circuit. - Ikk=0: Generator with no compound excitation. - Ikk<>0: Generator with compound
      excitation. Ikk is used to calculate the minimum steady-state short-circuit current for generators with
      compound excitation. (4.6.1.2 in IEC 60909-0:2001). Used only for single fed short circuit on a
      generator. (4.3.4.2. in IEC 60909-0:2001).
    mu: Factor to calculate the breaking current (Section 4.5.2.1 in IEC 60909-0). Used only for single fed short
      circuit on a generator (Section 4.3.4.2. in IEC 60909-0).
    x0: Zero sequence reactance of the synchronous machine.
    r0: Zero sequence resistance of the synchronous machine.
    x2: Negative sequence reactance.
    r2: Negative sequence resistance.
    r: Equivalent resistance (RG) of generator. RG is considered for the calculation of all currents, except for the
      calculation of the peak current ip. Used for short circuit data exchange according to IEC 60909.
    satDirectSubtransX: Direct-axis subtransient reactance saturated, also known as Xd`sat.
    satDirectSyncX: Direct-axes saturated synchronous reactance (xdsat); reciprocal of short-circuit ration. Used for
      short circuit data exchange, only for single fed short circuit on a generator. (4.3.4.2. in
      IEC 60909-0:2001).
    satDirectTransX: Saturated Direct-axis transient reactance. The attribute is primarily used for short circuit
      calculations according to ANSI.
    shortCircuitRotorType: Type of rotor, used by short circuit applications, only for single fed short circuit
      according to IEC 60909.
    voltageRegulationRange: Range of generator voltage regulation (PG in IEC 60909-0) used for calculation of the
      impedance correction factor KG defined in IEC 60909-0. This attribute is used to
      describe the operating voltage of the generating unit.
    operatingMode: Current mode of operation.
    referencePriority: Priority of unit for use as powerflow voltage phase angle reference bus selection. 0 = don t care
      (default) 1 = highest priority. 2 is less than 1 and so on.
    SynchronousMachineDynamics: Synchronous machine dynamics model used to describe dynamic behaviour of this
      synchronous machine.
    """

    InitialReactiveCapabilityCurve: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    maxQ: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    minQ: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    qPercent: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    type: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    earthing: bool = Field(
        default=False,
        in_profiles=[
            Profile.SC,
        ],
    )

    earthingStarPointR: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    earthingStarPointX: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    ikk: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    mu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    x0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    r0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    x2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    r2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    r: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    satDirectSubtransX: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    satDirectSyncX: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    satDirectTransX: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    shortCircuitRotorType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SC,
        ],
    )

    voltageRegulationRange: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    operatingMode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SSH,
        ],
    )

    referencePriority: int = Field(
        default=0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # SynchronousMachineDynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SC,
            Profile.SSH,
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEST5B(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type ST5B model. The type ST5B excitation system is a variation of the type ST1A model, with
      alternative overexcitation and underexcitation inputs and additional limits. The block diagram in the IEEE
      421.5 standard has input signal Vc and does not indicate the summation point with Vref. The implementation of
      the ExcIEEEST5B shall consider summation point with Vref. Reference: IEEE 421.5-2005, 7.5.

    kr: Regulator gain (KR) (> 0).  Typical value = 200.
    t1: Firing circuit time constant (T1) (>= 0).  Typical value = 0,004.
    kc: Rectifier regulation factor (KC) (>= 0).  Typical value = 0,004.
    vrmax: Maximum voltage regulator output (VRMAX) (> 0).  Typical value = 5.
    vrmin: Minimum voltage regulator output (VRMIN) (< 0).  Typical value = -4.
    tc1: Regulator lead time constant (TC1) (>= 0).  Typical value = 0,8.
    tb1: Regulator lag time constant (TB1) (>= 0).  Typical value = 6.
    tc2: Regulator lead time constant (TC2) (>= 0).  Typical value = 0,08.
    tb2: Regulator lag time constant (TB2) (>= 0).  Typical value = 0,01.
    toc1: OEL lead time constant (TOC1) (>= 0).  Typical value = 0,1.
    tob1: OEL lag time constant (TOB1) (>= 0).  Typical value = 2.
    toc2: OEL lead time constant (TOC2) (>= 0).  Typical value = 0,08.
    tob2: OEL lag time constant (TOB2) (>= 0).  Typical value = 0,08.
    tuc1: UEL lead time constant (TUC1) (>= 0).  Typical value = 2.
    tub1: UEL lag time constant (TUB1) (>= 0).  Typical value = 10.
    tuc2: UEL lead time constant (TUC2) (>= 0).  Typical value = 0,1.
    tub2: UEL lag time constant (TUB2) (>= 0).  Typical value = 0,05.
    """

    kr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    toc1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tob1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    toc2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tob2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tuc1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tub1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tuc2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tub2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class WindContQIEC(IdentifiedObject):
    """
    Q control model. Reference: IEC 61400-27-1:2015, 5.6.5.7.

    iqh1: Maximum reactive current injection during dip (iqh1). It is a type-dependent parameter.
    iqmax: Maximum reactive current injection (iqmax) (> WindContQIEC.iqmin). It is a type-dependent parameter.
    iqmin: Minimum reactive current injection (iqmin) (< WindContQIEC.iqmax). It is a type-dependent parameter.
    iqpost: Post fault reactive current injection (iqpost). It is a project-dependent parameter.
    kiq: Reactive power PI controller integration gain (KI,q). It is a type-dependent parameter.
    kiu: Voltage PI controller integration gain (KI,u). It is a type-dependent parameter.
    kpq: Reactive power PI controller proportional gain (KP,q). It is a type-dependent parameter.
    kpu: Voltage PI controller proportional gain (KP,u). It is a type-dependent parameter.
    kqv: Voltage scaling factor for UVRT current (Kqv). It is a project-dependent parameter.
    tpfiltq: Power measurement filter time constant (Tpfiltq) (>= 0). It is a type-dependent parameter.
    rdroop: Resistive component of voltage drop impedance (rdroop) (>= 0). It is a project-dependent parameter.
    tufiltq: Voltage measurement filter time constant (Tufiltq) (>= 0). It is a type-dependent parameter.
    tpost: Length of time period where post fault reactive power is injected (Tpost) (>= 0). It is a project-dependent
      parameter.
    tqord: Time constant in reactive power order lag (Tqord) (>= 0). It is a type-dependent parameter.
    udb1: Voltage deadband lower limit (udb1). It is a type-dependent parameter.
    udb2: Voltage deadband upper limit (udb2). It is a type-dependent parameter.
    umax: Maximum voltage in voltage PI controller integral term (umax) (> WindContQIEC.umin). It is a type-dependent
      parameter.
    umin: Minimum voltage in voltage PI controller integral term (umin) (< WindContQIEC.umax). It is a type-dependent
      parameter.
    uqdip: Voltage threshold for UVRT detection in Q control (uqdip). It is a type-dependent parameter.
    uref0: User-defined bias in voltage reference (uref0). It is a case-dependent parameter.
    windQcontrolModesType: Types of general wind turbine Q control modes (MqG).  It is a project-dependent parameter.
    windUVRTQcontrolModesType: Types of UVRT Q control modes (MqUVRT). It is a project-dependent parameter.
    xdroop: Inductive component of voltage drop impedance (xdroop) (>= 0). It is a project-dependent parameter.
    WindTurbineType3or4IEC: Wind turbine type 3 or type 4 model with which this reactive control model is associated.
    """

    iqh1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    iqmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    iqmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    iqpost: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kiq: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kiu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpq: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kqv: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpfiltq: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rdroop: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tufiltq: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpost: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tqord: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    udb1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    udb2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    umax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    umin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uqdip: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uref0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    windQcontrolModesType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    windUVRTQcontrolModesType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    xdroop: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3or4IEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class MeasurementValueSource(IdentifiedObject):
    """
    MeasurementValueSource describes the alternative sources updating a MeasurementValue. User conventions for how to
      use the MeasurementValueSource attributes are defined in IEC 61970-301.

    MeasurementValues: The MeasurementValues updated by the source.
    """

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # MeasurementValues : list = Field(default_factory=list, in_profiles = [Profile.OP, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class ExcIEEEST1AUELselectorKind(Base):
    """
    Types of connections for the UEL input used in ExcIEEEST1A.

    """

    # No attributes defined for this class.

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .DCConductingEquipment import DCConductingEquipment


@dataclass(config=DataclassConfig)
class DCLineSegment(DCConductingEquipment):
    """
    A wire or combination of wires not insulated from one another, with consistent electrical characteristics, used to
      carry direct current between points in the DC region of the power system.

    capacitance: Capacitance of the DC line segment. Significant for cables only.
    inductance: Inductance of the DC line segment. Negligible compared with DCSeriesDevice used for smoothing.
    resistance: Resistance of the DC line segment.
    length: Segment length for calculating line section capabilities.
    """

    capacitance: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    inductance: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    resistance: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    length: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IOPoint import IOPoint


@dataclass(config=DataclassConfig)
class MeasurementValue(IOPoint):
    """
    The current state for a measurement. A state value is an instance of a measurement from a specific source.
      Measurements can be associated with many state values, each representing a different source for the
      measurement.

    timeStamp: The time when the value was last updated.
    sensorAccuracy: The limit, expressed as a percentage of the sensor maximum, that errors will not exceed when the
      sensor is used under  reference conditions.
    MeasurementValueQuality: A MeasurementValue has a MeasurementValueQuality associated with it.
    MeasurementValueSource: A reference to the type of source that updates the MeasurementValue, e.g. SCADA, CCLink,
      manual, etc. User conventions for the names of sources are contained in the
      introduction to IEC 61970-301.
    """

    timeStamp: str = Field(
        default="",
        in_profiles=[
            Profile.OP,
        ],
    )

    sensorAccuracy: float = Field(
        default=0.0,
        in_profiles=[
            Profile.OP,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # MeasurementValueQuality : Optional[str] = Field(default=None, in_profiles = [Profile.OP, ])

    MeasurementValueSource: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.OP,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEAC4A(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type AC4A model. The model represents type AC4A alternator-supplied controlled-rectifier excitation
      system which is quite different from the other types of AC systems. This high initial response excitation
      system utilizes a full thyristor bridge in the exciter output circuit.  The voltage regulator controls the
      firing of the thyristor bridges. The exciter alternator uses an independent voltage regulator to control its
      output voltage to a constant value. These effects are not modelled; however, transient loading effects on the
      exciter alternator are included. Reference: IEEE 421.5-2005, 6.4.

    vimax: Maximum voltage regulator input limit (VIMAX) (> 0).  Typical value = 10.
    vimin: Minimum voltage regulator input limit (VIMIN) (< 0).  Typical value = -10.
    tc: Voltage regulator time constant (TC) (>= 0).  Typical value = 1.
    tb: Voltage regulator time constant (TB) (>= 0).  Typical value = 10.
    ka: Voltage regulator gain (KA) (> 0).  Typical value = 200.
    ta: Voltage regulator time constant (TA) (> 0).  Typical value = 0,015.
    vrmax: Maximum voltage regulator output (VRMAX) (> 0).  Typical value = 5,64.
    vrmin: Minimum voltage regulator output (VRMIN) (< 0).  Typical value = -4,53.
    kc: Rectifier loading factor proportional to commutating reactance (KC) (>= 0).  Typical value = 0.
    """

    vimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vimin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .OverexcitationLimiterDynamics import OverexcitationLimiterDynamics


@dataclass(config=DataclassConfig)
class OverexcLimIEEE(OverexcitationLimiterDynamics):
    """
    The over excitation limiter model is intended to represent the significant features of OELs necessary for some
      large-scale system studies. It is the result of a pragmatic approach to obtain a model that can be widely
      applied with attainable data from generator owners. An attempt to include all variations in the functionality
      of OELs and duplicate how they interact with the rest of the excitation systems would likely result in a level
      of application insufficient for the studies for which they are intended. Reference: IEEE OEL 421.5-2005, 9.

    itfpu: OEL timed field current limiter pickup level (ITFPU).  Typical value = 1,05.
    ifdmax: OEL instantaneous field current limit (IFDMAX).  Typical value = 1,5.
    ifdlim: OEL timed field current limit (IFDLIM).  Typical value = 1,05.
    hyst: OEL pickup/drop-out hysteresis (HYST).  Typical value = 0,03.
    kcd: OEL cooldown gain (KCD).  Typical value = 1.
    kramp: OEL ramped limit rate (KRAMP).  Unit = PU / s.  Typical value = 10.
    """

    itfpu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ifdmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ifdlim: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    hyst: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kcd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kramp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TurbineGovernorDynamics import TurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovHydroWEH(TurbineGovernorDynamics):
    """
    WoodwardTM electric hydro governor.  [Footnote: Woodward electric hydro governors are an example of suitable
      products available commercially. This information is given for the convenience of users of this document and
      does not constitute an endorsement by IEC of these products.]

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    rpg: Permanent droop for governor output feedback (R-Perm-Gate).
    rpp: Permanent droop for electrical power feedback (R-Perm-Pe).
    tpe: Electrical power droop time constant (Tpe) (>= 0).
    kp: Derivative control gain (Kp).
    ki: Derivative controller Integral gain (Ki).
    kd: Derivative controller derivative gain (Kd).
    td: Derivative controller time constant (Td) (>= 0).  Limits the derivative characteristic beyond a breakdown
      frequency to avoid amplification of high-frequency noise.
    tp: Pilot valve time lag time constant (Tp) (>= 0).
    tdv: Distributive valve time lag time constant (Tdv) (>= 0).
    tg: Value to allow the distribution valve controller to advance beyond the gate movement rate limit (Tg) (>= 0).
    gtmxop: Maximum gate opening rate (Gtmxop).
    gtmxcl: Maximum gate closing rate (Gtmxcl).
    gmax: Maximum gate position (Gmax) (> GovHydroWEH.gmin).
    gmin: Minimum gate position (Gmin) (< GovHydroWEH.gmax).
    dturb: Turbine damping factor (Dturb).  Unit = delta P (PU of MWbase) / delta speed (PU).
    tw: Water inertia time constant (Tw) (> 0).
    db: Speed deadband (db).
    dpv: Value to allow the pilot valve controller to advance beyond the gate limits (Dpv).
    dicn: Value to allow the integral controller to advance beyond the gate limits (Dicn).
    feedbackSignal: Feedback signal selection (Sw). true = PID output (if R-Perm-Gate = droop and R-Perm-Pe = 0) false =
      electrical power (if R-Perm-Gate = 0 and R-Perm-Pe = droop) or false = gate position (if
      R-Perm-Gate = droop and R-Perm-Pe = 0). Typical value = false.
    gv1: Gate 1 (Gv1).  Gate Position value for point 1 for lookup table representing water flow through the turbine as
      a function of gate position to produce steady state flow.
    gv2: Gate 2 (Gv2).  Gate Position value for point 2 for lookup table representing water flow through the turbine as
      a function of gate position to produce steady state flow.
    gv3: Gate 3 (Gv3).  Gate Position value for point 3 for lookup table representing water flow through the turbine as
      a function of gate position to produce steady state flow.
    gv4: Gate 4 (Gv4).  Gate Position value for point 4 for lookup table representing water flow through the turbine as
      a function of gate position to produce steady state flow.
    gv5: Gate 5 (Gv5).  Gate Position value for point 5 for lookup table representing water flow through the turbine as
      a function of gate position to produce steady state flow.
    fl1: Flowgate 1 (Fl1).  Flow value for gate position point 1 for lookup table representing water flow through the
      turbine as a function of gate position to produce steady state flow.
    fl2: Flowgate 2 (Fl2).  Flow value for gate position point 2 for lookup table representing water flow through the
      turbine as a function of gate position to produce steady state flow.
    fl3: Flowgate 3 (Fl3).  Flow value for gate position point 3 for lookup table representing water flow through the
      turbine as a function of gate position to produce steady state flow.
    fl4: Flowgate 4 (Fl4).  Flow value for gate position point 4 for lookup table representing water flow through the
      turbine as a function of gate position to produce steady state flow.
    fl5: Flowgate 5 (Fl5).  Flow value for gate position point 5 for lookup table representing water flow through the
      turbine as a function of gate position to produce steady state flow.
    fp1: Flow P1 (Fp1).  Turbine flow value for point 1 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp2: Flow P2 (Fp2).  Turbine flow value for point 2 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp3: Flow P3 (Fp3).  Turbine flow value for point 3 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp4: Flow P4 (Fp4).  Turbine flow value for point 4 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp5: Flow P5 (Fp5).  Turbine flow value for point 5 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp6: Flow P6 (Fp6).  Turbine flow value for point 6 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp7: Flow P7 (Fp7).  Turbine flow value for point 7 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp8: Flow P8 (Fp8).  Turbine flow value for point 8 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp9: Flow P9 (Fp9).  Turbine flow value for point 9 for lookup table representing PU mechanical power on machine MVA
      rating as a function of turbine flow.
    fp10: Flow P10 (Fp10).  Turbine flow value for point 10 for lookup table representing PU mechanical power on machine
      MVA rating as a function of turbine flow.
    pmss1: Pmss flow P1 (Pmss1).  Mechanical power output for turbine flow point 1 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss2: Pmss flow P2 (Pmss2).  Mechanical power output for turbine flow point 2 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss3: Pmss flow P3 (Pmss3).  Mechanical power output for turbine flow point 3 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss4: Pmss flow P4 (Pmss4).  Mechanical power output for turbine flow point 4 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss5: Pmss flow P5 (Pmss5).  Mechanical power output for turbine flow point 5 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss6: Pmss flow P6 (Pmss6).  Mechanical power output for turbine flow point 6 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss7: Pmss flow P7 (Pmss7).  Mechanical power output for turbine flow point 7 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss8: Pmss flow P8 (Pmss8).  Mechanical power output for turbine flow point 8 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss9: Pmss flow P9 (Pmss9).  Mechanical power output for turbine flow point 9 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    pmss10: Pmss flow P10 (Pmss10).  Mechanical power output for turbine flow point 10 for lookup table representing PU
      mechanical power on machine MVA rating as a function of turbine flow.
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rpg: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rpp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpe: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    td: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tdv: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tg: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gtmxop: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gtmxcl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dturb: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    db: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dpv: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dicn: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    feedbackSignal: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fl1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fl2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fl3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fl4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fl5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp7: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp8: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp9: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fp10: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss7: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss8: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss9: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmss10: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssIEEE2B(PowerSystemStabilizerDynamics):
    """
    IEEE 421.5-2005 type PSS2B power system stabilizer model. This stabilizer model is designed to represent a variety
      of dual-input stabilizers, which normally use combinations of power and speed or frequency to derive the
      stabilizing signal. Reference: IEEE 2B 421.5-2005, 8.2.

    inputSignal1Type: Type of input signal #1 (rotorAngularFrequencyDeviation, busFrequencyDeviation).  Typical value =
      rotorAngularFrequencyDeviation.
    inputSignal2Type: Type of input signal #2 (generatorElectricalPower).  Typical value = generatorElectricalPower.
    vsi1max: Input signal #1 maximum limit (Vsi1max) (> PssIEEE2B.vsi1min).  Typical value = 2.
    vsi1min: Input signal #1 minimum limit (Vsi1min) (< PssIEEE2B.vsi1max).  Typical value = -2.
    tw1: First washout on signal #1 (Tw1) (>= 0).  Typical value = 2.
    tw2: Second washout on signal #1 (Tw2) (>= 0).  Typical value = 2.
    vsi2max: Input signal #2 maximum limit (Vsi2max) (> PssIEEE2B.vsi2min).  Typical value = 2.
    vsi2min: Input signal #2 minimum limit (Vsi2min) (< PssIEEE2B.vsi2max).  Typical value = -2.
    tw3: First washout on signal #2 (Tw3) (>= 0).  Typical value = 2.
    tw4: Second washout on signal #2 (Tw4) (>= 0).  Typical value = 0.
    t1: Lead/lag time constant (T1) (>= 0).  Typical value = 0,12.
    t2: Lead/lag time constant (T2) (>= 0).  Typical value = 0,02.
    t3: Lead/lag time constant (T3) (>= 0).  Typical value = 0,3.
    t4: Lead/lag time constant (T4) (>= 0).  Typical value = 0,02.
    t6: Time constant on signal #1 (T6) (>= 0).  Typical value = 0.
    t7: Time constant on signal #2 (T7) (>= 0).  Typical value = 2.
    t8: Lead of ramp tracking filter (T8) (>= 0).  Typical value = 0,2.
    t9: Lag of ramp tracking filter (T9) (>= 0).  Typical value = 0,1.
    t10: Lead/lag time constant (T10) (>= 0).  Typical value = 0.
    t11: Lead/lag time constant (T11) (>= 0).  Typical value = 0.
    ks1: Stabilizer gain (Ks1).  Typical value = 12.
    ks2: Gain on signal #2 (Ks2).  Typical value = 0,2.
    ks3: Gain on signal #2 input before ramp-tracking filter (Ks3).  Typical value = 1.
    n: Order of ramp tracking filter (N).  Typical value = 1.
    m: Denominator order of ramp tracking filter (M).  Typical value = 5.
    vstmax: Stabilizer output maximum limit (Vstmax) (> PssIEEE2B.vstmin).  Typical value = 0,1.
    vstmin: Stabilizer output minimum limit (Vstmin) (< PssIEEE2B.vstmax).  Typical value = -0,1.
    """

    inputSignal1Type: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    inputSignal2Type: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsi1max: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsi1min: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsi2max: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsi2min: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t7: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t8: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t9: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t10: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t11: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    n: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    m: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vstmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vstmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class TopologicalNode(IdentifiedObject):
    """
    For a detailed substation model a topological node is a set of connectivity nodes that, in the current network
      state, are connected together through any type of closed switches, including  jumpers. Topological nodes
      change as the current network state changes (i.e., switches, breakers, etc. change state). For a planning
      model, switch statuses are not used to form topological nodes. Instead they are manually created or deleted in
      a model builder tool. Topological nodes maintained this way are also called "busses".

    BaseVoltage: The base voltage of the topological node.
    ConnectivityNodes: The connectivity nodes combine together to form this topological node.  May depend on the current
      state of switches in the network.
    ConnectivityNodeContainer: The connectivity node container to which the topological node belongs.
    Terminal: The terminals associated with the topological node.   This can be used as an alternative to the
      connectivity node path to terminal, thus making it unnecessary to model connectivity nodes in some
      cases.   Note that if connectivity nodes are in the model, this association would probably not be
      used as an input specification.
    ReportingGroup: The reporting group to which the topological node belongs.
    SvInjection: The injection flows state variables associated with the topological node.
    SvVoltage: The state voltage associated with the topological node.
    AngleRefTopologicalIsland: The island for which the node is an angle reference.   Normally there is one angle
      reference node for each island.
    TopologicalIsland: A topological node belongs to a topological island.
    """

    BaseVoltage: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.TP,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # ConnectivityNodes : list = Field(default_factory=list, in_profiles = [Profile.TP, ])

    ConnectivityNodeContainer: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.TP,
        ],
    )

    # *Association not used*
    # Type M:1..n in CIM  # pylint: disable-next=line-too-long
    # Terminal : list = Field(default_factory=list, in_profiles = [Profile.TP, ])

    ReportingGroup: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.TP,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # SvInjection : Optional[str] = Field(default=None, in_profiles = [Profile.SV, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # SvVoltage : Optional[str] = Field(default=None, in_profiles = [Profile.SV, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # AngleRefTopologicalIsland : Optional[str] = Field(default=None, in_profiles = [Profile.SV, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # TopologicalIsland : Optional[str] = Field(default=None, in_profiles = [Profile.SV, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.TP,
            Profile.SV,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssIEEE1A(PowerSystemStabilizerDynamics):
    """
    IEEE 421.5-2005 type PSS1A power system stabilizer model. PSS1A is the generalized form of a PSS with a single input
      signal.  Reference: IEEE 1A 421.5-2005, 8.1.

    inputSignalType: Type of input signal (rotorAngularFrequencyDeviation, generatorElectricalPower, or
      busFrequencyDeviation).  Typical value = rotorAngularFrequencyDeviation.
    a1: PSS signal conditioning frequency filter constant (A1).  Typical value = 0,061.
    a2: PSS signal conditioning frequency filter constant (A2).  Typical value = 0,0017.
    t1: Lead/lag time constant (T1) (>= 0).  Typical value = 0,3.
    t2: Lead/lag time constant (T2) (>= 0).  Typical value = 0,03.
    t3: Lead/lag time constant (T3) (>= 0).  Typical value = 0,3.
    t4: Lead/lag time constant (T4) (>= 0).  Typical value = 0,03.
    t5: Washout time constant (T5) (>= 0).  Typical value = 10.
    t6: Transducer time constant (T6) (>= 0).  Typical value = 0,01.
    ks: Stabilizer gain (Ks).  Typical value = 5.
    vrmax: Maximum stabilizer output (Vrmax) (> PssIEEE1A.vrmin).  Typical value = 0,05.
    vrmin: Minimum stabilizer output (Vrmin) (< PssIEEE1A.vrmax).  Typical value = -0,05.
    """

    inputSignalType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    a1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .Measurement import Measurement


@dataclass(config=DataclassConfig)
class Analog(Measurement):
    """
    Analog represents an analog Measurement.

    positiveFlowIn: If true then this measurement is an active power, reactive power or current with the convention that
      a positive value measured at the Terminal means power is flowing into the related
      PowerSystemResource.
    AnalogValues: The values connected to this measurement.
    LimitSets: A measurement may have zero or more limit ranges defined for it.
    """

    positiveFlowIn: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # AnalogValues : list = Field(default_factory=list, in_profiles = [Profile.OP, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # LimitSets : list = Field(default_factory=list, in_profiles = [Profile.OP, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssIEEE4B(PowerSystemStabilizerDynamics):
    """
    IEEE 421.5-2005 type PSS4B power system stabilizer. The PSS4B model represents a structure based on multiple working
      frequency bands. Three separate bands, respectively dedicated to the low-, intermediate- and high-frequency
      modes of oscillations, are used in this delta omega (speed input) PSS. There is an error in the in IEEE
      421.5-2005 PSS4B model: the Pe input should read -Pe. This implies that the input Pe needs to be multiplied by
      -1. Reference: IEEE 4B 421.5-2005, 8.4.  Parameter details: This model has 2 input signals. They have the
      following fixed types (expressed in terms of InputSignalKind values): the first one is of
      rotorAngleFrequencyDeviation type and the second one is of generatorElectricalPower type.

    bwh1: Notch filter 1 (high-frequency band): three dB bandwidth (Bwi).
    bwh2: Notch filter 2 (high-frequency band): three dB bandwidth (Bwi).
    bwl1: Notch filter 1 (low-frequency band): three dB bandwidth (Bwi).
    bwl2: Notch filter 2 (low-frequency band): three dB bandwidth (Bwi).
    kh: High band gain (KH).  Typical value = 120.
    kh1: High band differential filter gain (KH1).  Typical value = 66.
    kh11: High band first lead-lag blocks coefficient (KH11).  Typical value = 1.
    kh17: High band first lead-lag blocks coefficient (KH17).  Typical value = 1.
    kh2: High band differential filter gain (KH2).  Typical value = 66.
    ki: Intermediate band gain (KI).  Typical value = 30.
    ki1: Intermediate band differential filter gain (KI1).  Typical value = 66.
    ki11: Intermediate band first lead-lag blocks coefficient (KI11).  Typical value = 1.
    ki17: Intermediate band first lead-lag blocks coefficient (KI17).  Typical value = 1.
    ki2: Intermediate band differential filter gain (KI2).  Typical value = 66.
    kl: Low band gain (KL).  Typical value = 7.5.
    kl1: Low band differential filter gain (KL1).  Typical value = 66.
    kl11: Low band first lead-lag blocks coefficient (KL11).  Typical value = 1.
    kl17: Low band first lead-lag blocks coefficient (KL17).  Typical value = 1.
    kl2: Low band differential filter gain (KL2).  Typical value = 66.
    omeganh1: Notch filter 1 (high-frequency band): filter frequency (omegani).
    omeganh2: Notch filter 2 (high-frequency band): filter frequency (omegani).
    omeganl1: Notch filter 1 (low-frequency band): filter frequency (omegani).
    omeganl2: Notch filter 2 (low-frequency band): filter frequency (omegani).
    th1: High band time constant (TH1) (>= 0).  Typical value = 0,01513.
    th10: High band time constant (TH10) (>= 0).  Typical value = 0.
    th11: High band time constant (TH11) (>= 0).  Typical value = 0.
    th12: High band time constant (TH12) (>= 0).  Typical value = 0.
    th2: High band time constant (TH2) (>= 0).  Typical value = 0,01816.
    th3: High band time constant (TH3) (>= 0).  Typical value = 0.
    th4: High band time constant (TH4) (>= 0).  Typical value = 0.
    th5: High band time constant (TH5) (>= 0).  Typical value = 0.
    th6: High band time constant (TH6) (>= 0).  Typical value = 0.
    th7: High band time constant (TH7) (>= 0).  Typical value = 0,01816.
    th8: High band time constant (TH8) (>= 0).  Typical value = 0,02179.
    th9: High band time constant (TH9) (>= 0).  Typical value = 0.
    ti1: Intermediate band time constant (TI1) (>= 0).  Typical value = 0,173.
    ti10: Intermediate band time constant (TI10) (>= 0).  Typical value = 0.
    ti11: Intermediate band time constant (TI11) (>= 0).  Typical value = 0.
    ti12: Intermediate band time constant (TI12) (>= 0).  Typical value = 0.
    ti2: Intermediate band time constant (TI2) (>= 0).  Typical value = 0,2075.
    ti3: Intermediate band time constant (TI3) (>= 0).  Typical value = 0.
    ti4: Intermediate band time constant (TI4) (>= 0).  Typical value = 0.
    ti5: Intermediate band time constant (TI5) (>= 0).  Typical value = 0.
    ti6: Intermediate band time constant (TI6) (>= 0).  Typical value = 0.
    ti7: Intermediate band time constant (TI7) (>= 0).  Typical value = 0,2075.
    ti8: Intermediate band time constant (TI8) (>= 0).  Typical value = 0,2491.
    ti9: Intermediate band time constant (TI9) (>= 0).  Typical value = 0.
    tl1: Low band time constant (TL1) (>= 0).  Typical value = 1,73.
    tl10: Low band time constant (TL10) (>= 0).  Typical value = 0.
    tl11: Low band time constant (TL11) (>= 0).  Typical value = 0.
    tl12: Low band time constant (TL12) (>= 0).  Typical value = 0.
    tl2: Low band time constant (TL2) (>= 0).  Typical value = 2,075.
    tl3: Low band time constant (TL3) (>= 0).  Typical value = 0.
    tl4: Low band time constant (TL4) (>= 0).  Typical value = 0.
    tl5: Low band time constant (TL5) (>= 0).  Typical value = 0.
    tl6: Low band time constant (TL6) (>= 0).  Typical value = 0.
    tl7: Low band time constant (TL7) (>= 0).  Typical value = 2,075.
    tl8: Low band time constant (TL8) (>= 0).  Typical value = 2,491.
    tl9: Low band time constant (TL9) (>= 0).  Typical value = 0.
    vhmax: High band output maximum limit (VHmax) (> PssIEEE4B.vhmin).  Typical value = 0,6.
    vhmin: High band output minimum limit (VHmin) (< PssIEEE4V.vhmax).  Typical value = -0,6.
    vimax: Intermediate band output maximum limit (VImax) (> PssIEEE4B.vimin).  Typical value = 0,6.
    vimin: Intermediate band output minimum limit (VImin) (< PssIEEE4B.vimax).  Typical value = -0,6.
    vlmax: Low band output maximum limit (VLmax) (> PssIEEE4B.vlmin).  Typical value = 0,075.
    vlmin: Low band output minimum limit (VLmin) (< PssIEEE4B.vlmax).  Typical value = -0,075.
    vstmax: PSS output maximum limit (VSTmax) (> PssIEEE4B.vstmin).  Typical value = 0,15.
    vstmin: PSS output minimum limit (VSTmin) (< PssIEEE4B.vstmax).  Typical value = -0,15.
    """

    bwh1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    bwh2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    bwl1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    bwl2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh11: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh17: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki11: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki17: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl11: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl17: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    omeganh1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    omeganh2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    omeganl1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    omeganl2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th10: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th11: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th12: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th7: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th8: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th9: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti10: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti11: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti12: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti7: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti8: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti9: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl10: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl11: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl12: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl7: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl8: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl9: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vhmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vhmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vimin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vlmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vlmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vstmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vstmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class IdentifiedObject(Base):
    """
    This is a root class to provide common identification for all classes needing identification and naming attributes.

    description: The description is a free human readable text describing or naming the object. It may be non unique and
      may not correlate to a naming hierarchy.
    energyIdentCodeEic: The attribute is used for an exchange of the EIC code (Energy identification Code). The length
      of the string is 16 characters as defined by the EIC code. For details on EIC scheme
      please refer to ENTSO-E web site.
    mRID: Master resource identifier issued by a model authority. The mRID is unique within an exchange context. Global
      uniqueness is easily achieved by using a UUID, as specified in RFC 4122, for the mRID. The use of UUID
      is strongly recommended. For CIMXML data files in RDF syntax conforming to IEC 61970-552, the mRID is
      mapped to rdf:ID or rdf:about attributes that identify CIM object elements.
    name: The name is any free human readable and possibly non unique text naming the object.
    shortName: The attribute is used for an exchange of a human readable short name with length of the string 12
      characters maximum.
    DiagramObjects: The diagram objects that are associated with the domain object.
    """

    description: str = Field(
        default="",
        in_profiles=[
            Profile.TP,
            Profile.DL,
            Profile.EQBD,
            Profile.EQ,
            Profile.DY,
            Profile.OP,
        ],
    )

    energyIdentCodeEic: str = Field(
        default="",
        in_profiles=[
            Profile.TP,
            Profile.EQBD,
            Profile.EQ,
        ],
    )

    mRID: str = Field(
        default="",
        in_profiles=[
            Profile.TP,
            Profile.GL,
            Profile.DL,
            Profile.EQBD,
            Profile.EQ,
            Profile.SC,
            Profile.SV,
            Profile.SSH,
            Profile.DY,
            Profile.OP,
        ],
    )

    name: str = Field(
        default="",
        in_profiles=[
            Profile.TP,
            Profile.GL,
            Profile.DL,
            Profile.EQBD,
            Profile.EQ,
            Profile.SV,
            Profile.DY,
            Profile.OP,
        ],
    )

    shortName: str = Field(
        default="",
        in_profiles=[
            Profile.TP,
            Profile.EQBD,
            Profile.EQ,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # DiagramObjects : list = Field(default_factory=list, in_profiles = [Profile.DL, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.TP,
            Profile.GL,
            Profile.DL,
            Profile.EQBD,
            Profile.EQ,
            Profile.SC,
            Profile.SV,
            Profile.SSH,
            Profile.DY,
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .SynchronousMachineDetailed import SynchronousMachineDetailed


@dataclass(config=DataclassConfig)
class SynchronousMachineTimeConstantReactance(SynchronousMachineDetailed):
    """
    Synchronous machine detailed modelling types are defined by the combination of the attributes
      SynchronousMachineTimeConstantReactance.modelType and SynchronousMachineTimeConstantReactance.rotorType.
      Parameter details:  The "p" in the time-related attribute names is a substitution for a "prime" in the usual
      parameter notation, e.g. tpdo refers to T'do. The parameters used for models expressed in time constant
      reactance form include:  - RotatingMachine.ratedS (MVAbase); - RotatingMachineDynamics.damping (D); -
      RotatingMachineDynamics.inertia (H); - RotatingMachineDynamics.saturationFactor (S1); -
      RotatingMachineDynamics.saturationFactor120 (S12); - RotatingMachineDynamics.statorLeakageReactance (Xl); -
      RotatingMachineDynamics.statorResistance (Rs); - SynchronousMachineTimeConstantReactance.ks (Ks); -
      SynchronousMachineDetailed.saturationFactorQAxis (S1q); - SynchronousMachineDetailed.saturationFactor120QAxis
      (S12q); - SynchronousMachineDetailed.efdBaseRatio; - SynchronousMachineDetailed.ifdBaseType; - .xDirectSync
      (Xd); - .xDirectTrans (X'd); - .xDirectSubtrans (X''d); - .xQuadSync (Xq); - .xQuadTrans (X'q); -
      .xQuadSubtrans (X''q); - .tpdo (T'do); - .tppdo (T''do); - .tpqo (T'qo); - .tppqo (T''qo); - .tc.

    rotorType: Type of rotor on physical machine.
    modelType: Type of synchronous machine model used in dynamic simulation applications.
    ks: Saturation loading correction factor (Ks) (>= 0).  Used only by type J model.  Typical value = 0.
    xDirectSync: Direct-axis synchronous reactance (Xd) (>= SynchronousMachineTimeConstantReactance.xDirectTrans). The
      quotient of a sustained value of that AC component of armature voltage that is produced by the
      total direct-axis flux due to direct-axis armature current and the value of the AC component of
      this current, the machine running at rated speed.  Typical value = 1,8.
    xDirectTrans: Direct-axis transient reactance (unsaturated) (X`d) (>=
      SynchronousMachineTimeConstantReactance.xDirectSubtrans).  Typical value = 0,5.
    xDirectSubtrans: Direct-axis subtransient reactance (unsaturated) (X``d) (>
      RotatingMachineDynamics.statorLeakageReactance).  Typical value = 0,2.
    xQuadSync: Quadrature-axis synchronous reactance (Xq) (>= SynchronousMachineTimeConstantReactance.xQuadTrans). The
      ratio of the component of reactive armature voltage, due to the quadrature-axis component of
      armature current, to this component of current, under steady state conditions and at rated
      frequency.  Typical value = 1,6.
    xQuadTrans: Quadrature-axis transient reactance (X`q) (>= SynchronousMachineTimeConstantReactance.xQuadSubtrans).
      Typical value = 0,3.
    xQuadSubtrans: Quadrature-axis subtransient reactance (X``q) (> RotatingMachineDynamics.statorLeakageReactance).
      Typical value = 0,2.
    tpdo: Direct-axis transient rotor time constant (T`do) (> SynchronousMachineTimeConstantReactance.tppdo).  Typical
      value = 5.
    tppdo: Direct-axis subtransient rotor time constant (T``do) (> 0).  Typical value = 0,03.
    tpqo: Quadrature-axis transient rotor time constant (T`qo) (> SynchronousMachineTimeConstantReactance.tppqo).
      Typical value = 0,5.
    tppqo: Quadrature-axis subtransient rotor time constant (T``qo) (> 0). Typical value = 0,03.
    tc: Damping time constant for `Canay` reactance (>= 0).  Typical value = 0.
    """

    rotorType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    modelType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xDirectSync: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xDirectTrans: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xDirectSubtrans: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xQuadSync: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xQuadTrans: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xQuadSubtrans: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpdo: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tppdo: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpqo: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tppqo: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }
